/*************************************************************************

    File: fol2latex.pl
    Copyright (C) 2004 Patrick Blackburn & Johan Bos

    This file is part of BB1, version 1.1 (June 2005).

    BB1 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    BB1 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with BB1; if not, write to the Free Software Foundation, Inc., 
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*************************************************************************/


fol2file(F):-
    open('test.tex',write,Stream),
    write(Stream,'\\documentstyle{article}'),
    nl(Stream),
    write(Stream,'\\begin{document}'),
    nl(Stream),
    numbervars(F,0,_),
    fol2latex(F,Stream),
    nl(Stream),
    write(Stream,'\\end{document}'),
    nl(Stream),
    close(Stream).

fol2latex(F):-
    fol2latex(F,user).


fol2latex(some(X,F),Stream):- !,
    format(Stream,'$\\exists$~p',[X]),
    fol2latex(F,Stream).

fol2latex(all(X,F),Stream):- !,
    format(Stream,'$\\forall$~p',[X]),
    fol2latex(F,Stream).

fol2latex(lam(X,F),Stream):- !,
    format(Stream,'$\\lambda$~p.',[X]),
    fol2latex(F,Stream).

fol2latex(que(X,F1,F2),Stream):- !,
    format(Stream,'?~p(',[X]),
    fol2latex(F1,Stream),
    write(Stream,','),
    fol2latex(F2,Stream),
    write(Stream,')').

fol2latex(and(F1,F2),Stream):- !,
    write(Stream,'('),
    fol2latex(F1,Stream),
    write(Stream,' $\\land$ '),
    fol2latex(F2,Stream),
    write(Stream,')').

fol2latex(imp(F1,F2),Stream):- !,
    write(Stream,'('),
    fol2latex(F1,Stream),
    write(Stream,' $\\to$ '),
    fol2latex(F2,Stream),
    write(Stream,')').

fol2latex(or(F1,F2),Stream):- !,
    write(Stream,'('),
    fol2latex(F1,Stream),
    write(Stream,' $\\lor$ '),
    fol2latex(F2,Stream),
    write(Stream,')').

fol2latex(app(F1,F2),Stream):- !,
    write(Stream,'('),
    fol2latex(F1,Stream),
    write(Stream,'@'),
    fol2latex(F2,Stream),
    write(Stream,')').

fol2latex(leq(X,Y),Stream):- !,
    format(Stream,'~p$\\leq$~p',[X,Y]).

fol2latex(not(F),Stream):- !,
    write(Stream,'$\\neg$'),
    fol2latex(F,Stream).

fol2latex(pred1(L,S,X),Stream):- !,
    format(Stream,'~p:\\textsc{~p}(~p)',[L,S,X]).

fol2latex(pred2(L,S,X,Y),Stream):- !,
    format(Stream,'~p:\\textsc{~p}(~p,~p)',[L,S,X,Y]).

fol2latex(eq(L,X,Y),Stream):- !,
    format(Stream,'~p:~p=~p',[L,X,Y]).

fol2latex(F,Stream):-
    F =.. [Symbol,Arg],
    format(Stream,'\\textsc{~p}(~p)',[Symbol,Arg]).

fol2latex(F,Stream):-
    F =.. [Symbol,Arg1,Arg2],
    format(Stream,'\\textsc{~p}(~p,~p)',[Symbol,Arg1,Arg2]).

fol2latex(F,Stream):-
    F =.. [Symbol,Arg1,Arg2,Arg3],
    format(Stream,'\\textsc{~p}(~p,~p,~p)',[Symbol,Arg1,Arg2,Arg3]).
